function model = weaklearner_perc(data)
% WEAKLEARNER_PERC Produce linear classifier using weighted data.
%
% Synopsis:
%  model = weaklearner_perc(data)
%
% Description:
%  This function produces a binary classifier which assigns
%  input vector x to classes [1,2] based on linear discriminant function.
%  The linear discr. function is learned by the batch perceptron algorithm.
%  This weak learner can be used with the AdaBoost classifier
%  (see 'help adaboost').
%  
% Input:
%  data [struct] Training data:
%   .X [dim x num_data] Training vectors.
%   .y [1 x num_data] Binary labels (1 or 2).
%   .D [1 x num_data] Weights of training vectors (optional).
%    If not given then D is set to be uniform distribution.
% 
% Output:
%  see the output of SMO function
%
% Example:
%  help adaboost
%
% See also: 
%  ADABOOST, ADACLASS.
% 

% About: Statistical Pattern Recognition Toolbox
% (C) 1999-2004, Written by Vojtech Franc and Vaclav Hlavac
% <a href="http://www.cvut.cz">Czech Technical University Prague</a>
% <a href="http://www.feld.cvut.cz">Faculty of Electrical Engineering</a>
% <a href="http://cmp.felk.cvut.cz">Center for Machine Perception</a>

% Modifications:
% 19-feb-2010, PP, created from weaklearner.m
% 25-aug-2004, VF
% 11-aug-2004, VF

[dim,N] = size(data.X);
D = ones(1,N)/N;
if isfield(data,'D'), 
    D = data.D(:)';
end

x = [data.X; ones(1,N)];
y = data.y;

% Invert points in class 2
x(:,y~=1) = -x(:,y~=1);
w = x(:,1);

minwerr = inf;

for i = 1:200,
    pred = (w'*x);
    ind = (pred < 0);
    werr = 0;
    werr = sum( D(ind) );
    if werr < minwerr,
        bestw = w;
        minwerr = werr;
    end
    if werr == 0, break; end
    deltaw = x(:,ind) * D(ind)';
    w = w + deltaw;
end

model.W = bestw(1:end-1);
model.b = bestw(end);
model.fun = 'linclass';

end